#! /usr/bin/env sh

dry_run=false

usage () {
cat <<_END_OF_USAGE
Usage: $0

Install build dependencies for OVS.

Options:
help | -h | --help  Print this message.
-n | --dry-run      Print intended actions without executing. [$dry_run]
_END_OF_USAGE
exit 0
}

for arg ; do
case "$arg" in
help|--help|-h) DO_USAGE=y ;;
-n|--dry-run) dry_run=true ;;
*) echo "unknown option $arg"; FATAL=y ;;
esac
done

[ "$FATAL" ] && usage
[ "$DO_USAGE" ] && usage

x() {
    if test x$dry_run = xtrue
    then echo "$*"
    else eval "$*"
    fi
}

cmd_exist() { command -v $1 1> /dev/null; }
pkg_update() {
    if   cmd_exist apt; then     x apt update
    elif cmd_exist apt-get; then x apt-get update
    elif cmd_exist dnf; then     x dnf update
    elif cmd_exist yum; then     x yum update
    fi
}
pkg_install() {
    if   cmd_exist apt; then     x apt install -y $*
    elif cmd_exist apt-get; then x apt-get install -y $*
    elif cmd_exist dnf; then     x dnf install -y $*
    elif cmd_exist yum; then     x yum install -y $*
    fi
}

if cmd_exist apt || cmd_exist apt-get; then
    FLAVOR=deb
elif cmd_exist dnf || cmd_exist yum; then
    FLAVOR=rpm
else
    echo "No supported packaging system found, cancelling."
    exit
fi

rpm_packages() {
    cat<<EOL
autoconf automake openssl-devel libtool unbound-devel unbound
libpcap-devel libbsd-devel elfutils-libelf-devel libunwind-devel
groff graphviz rpmdevtools nc curl libreswan network-scripts
checkpolicy selinux-policy-devel libcap-ng-devel ethtool net-tools lftp
python3-rpm-macros
python3-devel
python3-zope-interface
python3-prometheus_client
python3-sphinx
python3-enchant
EOL
}

deb_packages() {
    cat<<EOL
autoconf automake libtool gcc bc lftp graphviz
libunbound-dev libunwind-dev
libssl-dev libelf-dev
libnuma-dev libpcap-dev
linux-headers-$(uname -r) build-essential fakeroot devscripts equivs
python3-all
python3-sphinx
python3-twisted
python3-prometheus-client
python3-zope.interface
python3-enchant
python3-pyelftools
EOL
}

packages=$(eval ${FLAVOR}_packages | xargs)
pkg_update
pkg_install "$packages"
